package java.security;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.Hashtable;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.Collections;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.HashMap;
import java.io.ObjectStreamField;
import java.util.List;
import java.util.Map;
import java.io.Serializable;

final class UnresolvedPermissionCollection extends PermissionCollection implements Serializable
{
    private transient Map<String, List<UnresolvedPermission>> perms;
    private static final long serialVersionUID = -7176153071733132400L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public UnresolvedPermissionCollection() {
        this.perms = new HashMap(11);
    }
    
    @Override
    public void add(Permission obj) {
        if (!(obj instanceof UnresolvedPermission)) {
            throw new IllegalArgumentException("invalid permission: " + obj);
        }
        UnresolvedPermission unresolvedPermission = (UnresolvedPermission)obj;
        List<UnresolvedPermission> list;
        synchronized (this) {
            list = (List)this.perms.get(unresolvedPermission.getName());
            if (list == null) {
                list = new ArrayList();
                this.perms.put(unresolvedPermission.getName(), list);
            }
        }
        synchronized (list) {
            list.add(unresolvedPermission);
        }
        return;
    }
    
    List<UnresolvedPermission> getUnresolvedPermissions(Permission permission) {
        synchronized (this) {
            return (List)this.perms.get(permission.getClass().getName());
        }
    }
    
    @Override
    public boolean implies(Permission permission) {
        return false;
    }
    
    @Override
    public Enumeration<Permission> elements() {
        ArrayList<Permission> c = new ArrayList();
        synchronized (this) {
            for (List list : this.perms.values()) {
                synchronized (list) {
                    c.addAll(list);
                }
            }
        }
        return Collections.enumeration(c);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable<Object, Object> hashtable = new Hashtable(this.perms.size() * 2);
        synchronized (this) {
            for (Map.Entry entry : this.perms.entrySet()) {
                List<Object> c = (List)entry.getValue();
                Vector value = new Vector(c.size());
                synchronized (c) {
                    value.addAll(c);
                }
                hashtable.put(entry.getKey(), value);
            }
        }
        objectOutputStream.putFields().put("permissions", hashtable);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Hashtable<Object, Object> hashtable = (Hashtable)objectInputStream.readFields().get("permissions", null);
        this.perms = new HashMap(hashtable.size() * 2);
        for (Map.Entry entry : hashtable.entrySet()) {
            Vector<Object> vector = (Vector)entry.getValue();
            ArrayList o = new ArrayList(vector.size());
            o.addAll(vector);
            this.perms.put(entry.getKey(), o);
        }
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("permissions", Hashtable.class) };
    }
}
